///////////////////////////////////////////////////////////////////////////////
//                                                                           //
// Silicon Graphics                                                          //
//                                                                           //
// hash_multimap < Key, Data, HashFcn, EqualKey, Alloc >                     //
//                                                                           //
// Hash_multimap is a Hashed Associative Container that associates objects   //
// of type Key with objects of type Data. Hash_multimap is a Pair            //
// Associative Container, meaning that its value type is                     //
// pair < const Key, Data >. It is also a Multiple Associative Container,    //
// meaning that there is no limit on the number of elements whose keys may   //
// compare equal using EqualKey.                                             //
//                                                                           //
// Looking up an element in a hash_multimap by its key is efficient, so      //
// hash_multimap is useful for "dictionaries" where the order of elements is //
// irrelevant. If it is important for the elements to be in a particular     //
// order, however, then multimap is more appropriate.                        //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



///////////////////////////////////////////////////////////////////////////////
//                                                                           //
// Jan Knepper 19980821                                                      //
//                                                                           //
// Above comments copied form SGI's document stl_doc/hash_multimap.html      //
// Code copied from the same document. Reformated a little so it conforms to //
// my personal standards and hopefully improves readability.                 //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////



#include <stdio.h>
#include <string.h>

#include <hash_map.h>



struct  eqstr
{
   bool                 operator () ( const char *  s1, const char *  s2 ) const
   {
      return ( strcmp ( s1, s2 ) == 0 );
   }
};



typedef hash_multimap < const char *, int, hash < const char * >, eqstr >     map_type;



void  lookup ( const map_type &  Map, const char *  str )
{
   cout << str << ": ";

   pair < map_type :: const_iterator, map_type :: const_iterator >   p = Map.equal_range ( str );

   for ( map_type :: const_iterator  i = p.first ; i != p.second ; ++i )
      cout << ( *i ).second << " ";
   cout << endl;
}



int  main ()
{
   map_type   M;

   M.insert ( map_type :: value_type ( "H",   1 ) );
   M.insert ( map_type :: value_type ( "H",   2 ) );
   M.insert ( map_type :: value_type ( "C",  12 ) );
   M.insert ( map_type :: value_type ( "C",  13 ) );
   M.insert ( map_type :: value_type ( "O",  16 ) );
   M.insert ( map_type :: value_type ( "O",  17 ) );
   M.insert ( map_type :: value_type ( "O",  18 ) );
   M.insert ( map_type :: value_type ( "I", 127 ) );

   lookup ( M, "I"  );
   lookup ( M, "O"  );
   lookup ( M, "Rn" );

   return (  0 );
}
